@@ -1,3 +1,4 @@
+import dateutil
 import copy
 import datetime
 import traceback
@@ -54,7 +55,7 @@ def _ts_add_previous_rows(df, historical_columns, window):
     return df
 
 
-def _ts_add_previous_target(df, predict_columns, nr_predictions, window):
+def _ts_add_previous_target(df, predict_columns, nr_predictions, window, mode):
     for target_column in predict_columns:
         previous_target_values = list(df[target_column])
         del previous_target_values[-1]
@@ -71,13 +72,35 @@ def _ts_add_previous_target(df, predict_columns, nr_predictions, window):
         for timestep_index in range(1, nr_predictions):
             next_target_value_arr = list(df[target_column])
             for del_index in range(0, min(timestep_index, len(next_target_value_arr))):
-                del next_target_value_arr[del_index]
-                next_target_value_arr.append(0)
-            # @TODO: Maybe ignore the rows with `None` next targets for training
+                del next_target_value_arr[0]
+                next_target_value_arr.append(None)
             df[f'{target_column}_timestep_{timestep_index}'] = next_target_value_arr
+
+    # drop rows with incomplete target info.
+    if mode == 'learn':
+        for target_column in predict_columns:
+            for col in [f'{target_column}_timestep_{i}' for i in range(1, nr_predictions)]:
+                if 'make_predictions' not in df.columns:
+                    df['make_predictions'] = True
+                df.loc[df[col].isna(), ['make_predictions']] = False
+
     return df
 
 
+def _ts_infer_next_row(df, ob, last_index):
+    last_row = df.iloc[[-1]].copy()
+    if df.shape[0] > 1:
+        butlast_row = df.iloc[[-2]]
+        delta = (last_row[ob].values - butlast_row[ob].values).flatten()[0]
+    else:
+        delta = 1
+    last_row.original_index = None
+    last_row.index = [last_index + 1]
+    last_row['make_predictions'] = True
+    last_row[ob] += delta
+    return df.append(last_row)
+
+
 class LightwoodBackend:
     def __init__(self, transaction):
         self.transaction = transaction
@@ -86,16 +109,21 @@ class LightwoodBackend:
         # Ideally this will go away soon but still a necessity for now
         self.nn_mixer_only = False
 
-    def _ts_reshape(self, original_df):
+    def _ts_reshape(self, original_df, mode='learn'):
         original_df = copy.deepcopy(original_df)
         gb_arr = self.transaction.lmd['tss']['group_by'] if self.transaction.lmd['tss']['group_by'] is not None else []
         ob_arr = self.transaction.lmd['tss']['order_by']
         window = self.transaction.lmd['tss']['window']
 
+        if 'make_predictions' in original_df.columns:
+            index = original_df[original_df['make_predictions'].map({'True': True, 'False': False, True: True, False: False}) == True]
+            infer_mode = index.shape[0] == 0  # condition to trigger: make_predictions is set to False everywhere
+        else:
+            infer_mode = False
         original_index_list = []
         idx = 0
         for row in original_df.itertuples():
-            if _make_pred(row):
+            if _make_pred(row) or infer_mode:
                 original_index_list.append(idx)
                 idx += 1
             else:
@@ -119,9 +147,9 @@ class LightwoodBackend:
                 else:
                     if self.transaction.lmd['stats_v2'][col]['typing']['data_type'] == DATA_TYPES.DATE:
                         try:
-                            row[col] = datetime.datetime.strptime(
+                            row[col] = dateutil.parser.parse(
                                 row[col],
-                                self.transaction.lmd['stats_v2'][col]['additional_info']['date_fmt']
+                                **self.transaction.lmd.get('dateutil_parser_kwargs_per_column', {}).get(col, {})
                             )
                         except (TypeError, ValueError):
                             pass
@@ -141,6 +169,13 @@ class LightwoodBackend:
         else:
             df_arr = [original_df]
 
+        last_index = original_df['original_index'].max()
+        for i, subdf in enumerate(df_arr):
+            if 'make_predictions' in subdf.columns and mode == 'predict':
+                if infer_mode:
+                    df_arr[i] = _ts_infer_next_row(subdf, ob_arr, last_index)
+                    last_index += 1
+
         if len(original_df) > 500:
             nr_procs = get_nr_procs(self.transaction.lmd.get('max_processes', None),
                                     self.transaction.lmd.get('max_per_proc_usage', None),
@@ -152,7 +187,7 @@ class LightwoodBackend:
             df_arr = pool.map(partial(_ts_order_col_to_cell_lists, historical_columns=ob_arr + self.transaction.lmd['tss']['historical_columns']), df_arr)
             df_arr = pool.map(partial(_ts_add_previous_rows, historical_columns=ob_arr + self.transaction.lmd['tss']['historical_columns'], window=window), df_arr)
             if self.transaction.lmd['tss']['use_previous_target']:
-                df_arr = pool.map(partial(_ts_add_previous_target, predict_columns=self.transaction.lmd['predict_columns'], nr_predictions=self.nr_predictions, window=window), df_arr)
+                df_arr = pool.map(partial(_ts_add_previous_target, predict_columns=self.transaction.lmd['predict_columns'], nr_predictions=self.nr_predictions, window=window, mode=mode), df_arr)
             pool.close()
             pool.join()
         else:
@@ -161,7 +196,7 @@ class LightwoodBackend:
                 df_arr[i] = _ts_order_col_to_cell_lists(df_arr[i], historical_columns=ob_arr + self.transaction.lmd['tss']['historical_columns'])
                 df_arr[i] = _ts_add_previous_rows(df_arr[i], historical_columns=ob_arr + self.transaction.lmd['tss']['historical_columns'], window=window)
                 if self.transaction.lmd['tss']['use_previous_target']:
-                    df_arr[i] = _ts_add_previous_target(df_arr[i], predict_columns=self.transaction.lmd['predict_columns'], nr_predictions=self.nr_predictions, window=window)
+                    df_arr[i] = _ts_add_previous_target(df_arr[i], predict_columns=self.transaction.lmd['predict_columns'], nr_predictions=self.nr_predictions, window=window, mode=mode)
 
         combined_df = pd.concat(df_arr)
 
@@ -184,17 +219,23 @@ class LightwoodBackend:
 
         if df_gb_map is None:
             for _, row in combined_df.iterrows():
-                timeseries_row_mapping[idx] = int(row['original_index']) if row['original_index'] is not None and not np.isnan(row['original_index']) else None
+                if not infer_mode:
+                    timeseries_row_mapping[idx] = int(row['original_index']) if row['original_index'] is not None and not np.isnan(row['original_index']) else None
+                else:
+                    timeseries_row_mapping[idx] = idx
                 idx += 1
         else:
             for gb in df_gb_map:
                 for _, row in df_gb_map[gb].iterrows():
-                    timeseries_row_mapping[idx] = int(row['original_index']) if row['original_index'] is not None and not np.isnan(row['original_index']) else None
+                    if not infer_mode:
+                        timeseries_row_mapping[idx] = int(row['original_index']) if row['original_index'] is not None and not np.isnan(row['original_index']) else None
+                    else:
+                        timeseries_row_mapping[idx] = idx
+
                     idx += 1
 
         del combined_df['original_index']
 
-
         return combined_df, secondary_type_dict, timeseries_row_mapping, df_gb_map
 
     def _create_lightwood_config(self, secondary_type_dict):
@@ -470,13 +511,17 @@ class LightwoodBackend:
 
                 if self.transaction.lmd['tss']['is_timeseries']:
                     validation_df = self.transaction.input_data.validation_df[self.transaction.input_data.validation_df['make_predictions'] == True]
+                    ts_window = self.transaction.lmd['tss'].get('window', 0)
+                else:
+                    ts_window = None
 
                 validation_accuracy = evaluate_accuracy(
                     validation_predictions,
                     validation_df,
                     self.transaction.lmd['stats_v2'],
                     self.transaction.lmd['predict_columns'],
-                    backend=self
+                    backend=self,
+                    ts_window=ts_window
                 )
 
                 predictors_and_accuracies.append((
@@ -529,7 +574,7 @@ class LightwoodBackend:
 
         df_gb_map = None
         if self.transaction.lmd['tss']['is_timeseries']:
-            df, _, timeseries_row_mapping, df_gb_map = self._ts_reshape(df)
+            df, _, timeseries_row_mapping, df_gb_map = self._ts_reshape(df, mode='predict')
 
         if df_gb_map is None:
             df_gb_map = {'': df}
@@ -554,6 +599,7 @@ class LightwoodBackend:
             predictions = self.predictor.predict(when_data=run_df)
 
             # cache run_df to avoid duplicate reshaping in analysis phase
+            # also used in streaming mode to retrieve newly added rows per group
             if mode == 'validate':
                 self.transaction.input_data.cached_val_df = run_df
             elif mode == 'predict':
