@@ -1,3 +1,5 @@
+import random
+import dateutil
 import string
 import numpy as np
 import imghdr
@@ -27,8 +29,33 @@ from mindsdb_native.libs.helpers.text_helpers import (
 from mindsdb_native.libs.phases.base_module import BaseModule
 from mindsdb_native.libs.helpers.stats_helpers import sample_data
 from mindsdb_native.libs.helpers.mp_helpers import get_nr_procs
+from mindsdb_native.libs.helpers.date_helpers import DATE_ORDER_KWARGS
 
-import flair
+# DATE_FMTS = [
+#     '%Y-%m-%d',
+#     '%Y/%m/%d',
+#     '%d.%m.%Y',
+#     '%Y/%m',
+#     '%Y-%m',
+#     '%d/%m/%Y',
+#     '%m/%d/%Y',
+#     '%m/%d/%y',
+# ]
+
+# DATETIME_FMTS = [
+#     '%Y-%m-%d %H:%M:%S',
+#     '%Y-%m-%d %H:%M:%S.%f',
+
+#     '%Y/%m/%d %H:%M:%S',
+#     '%Y/%m/%d %H:%M:%S.%f',
+
+#     '%d.%m.%Y %H:%M:%S',
+#     '%d.%m.%Y %H:%M:%S.%f',
+
+#     '%Y-%m-%dT%H:%M:%S.%f%Z',
+#     '%Y-%m-%dT%H:%M:%S.%f%z',
+#     '%Y-%m-%dT%H:%M:%S.%f'
+# ]
 
 
 def get_file_subtype_if_exists(path):
@@ -57,7 +84,7 @@ def get_number_subtype(string):
         return None
 
 
-def count_data_types_in_column(data, date_fmts, datetime_fmts):
+def count_data_types_in_column(data, lmd, col_name):
     type_counts = Counter()
     subtype_counts = Counter()
     additional_info = {}
@@ -101,33 +128,27 @@ def count_data_types_in_column(data, date_fmts, datetime_fmts):
         return type_guess, subtype_guess
 
     def type_check_date(element):
-        # date?
-        for fmt in date_fmts:
-            try:
-                datetime.datetime.strptime(element, fmt)
-            except Exception:
-                pass
-            else:
-                additional_info['date_fmt'] = fmt
-                return DATA_TYPES.DATE, DATA_SUBTYPES.DATE
+        type_guess, subtype_guess = None, None
+        try:
+            dt = dateutil.parser.parse(element, **lmd.get('dateutil_parser_kwargs_per_column', {}).get(col_name, {}))
 
-        # date + time?
-        for fmt in datetime_fmts:
-            try:
-                datetime.datetime.strptime(element, fmt)
-            except Exception:
-                pass
+            # Not accurate 100% for a single datetime str,
+            # but should work in aggregate
+            if dt.hour == 0 and dt.minute == 0 and \
+                dt.second == 0 and len(element) <= 16:
+                subtype_guess = DATA_SUBTYPES.DATE
             else:
-                additional_info['date_fmt'] = fmt
-                return DATA_TYPES.DATE, DATA_SUBTYPES.TIMESTAMP
-
-        return None, None
+                subtype_guess = DATA_SUBTYPES.TIMESTAMP
+            type_guess = DATA_TYPES.DATE
+        except ValueError:
+            pass
+        return type_guess, subtype_guess
 
     type_checkers = [type_check_numeric,
                      type_check_sequence,
                      type_check_file,
                      type_check_date]
-    for element in [str(x) for x in data]:
+    for element in data:
         for type_checker in type_checkers:
             type_guess, subtype_guess = type_checker(element)
             if type_guess is not None:
@@ -140,6 +161,7 @@ def count_data_types_in_column(data, date_fmts, datetime_fmts):
 
     return type_counts, subtype_counts, additional_info
 
+
 def get_column_data_type(arg_tup, lmd):
     """
     Provided the column data, define its data type and data subtype.
@@ -179,11 +201,7 @@ def get_column_data_type(arg_tup, lmd):
         subtype_dist[DATA_SUBTYPES.MULTIPLE] = len(data)
         return curr_data_type, curr_data_subtype, type_dist, subtype_dist, additional_info, warn, info
 
-    type_dist, subtype_dist, new_additional_info = count_data_types_in_column(
-        data,
-        date_fmts=lmd.get('date_fmts') or ['%Y-%m-%d', '%Y/%m/%d', '%d.%m.%Y', '%Y/%m', '%Y-%m'],
-        datetime_fmts=lmd.get('datetime_fmts') or ['%Y-%m-%d %H:%M:%S', '%Y/%m/%d %H:%M:%S', '%d.%m.%Y %H:%M:%S', '%Y-%m-%dT%H:%M:%S.%f%Z', '%Y-%m-%dT%H:%M:%S.%f%z', '%Y-%m-%dT%H:%M:%S.%f']
-    )
+    type_dist, subtype_dist, new_additional_info = count_data_types_in_column(data, lmd, col_name)
 
     if new_additional_info:
         additional_info.update(new_additional_info)
